/**
 * @file
 * Behaviours for Form element config.
 */

var qtoolsCommonFormElementQtoolsCommonConfig = {};

(function (Drupal, $, once) {

  var plugin = qtoolsCommonFormElementQtoolsCommonConfig;

  /**
   * Handle all prices placeholders in given context and run service request.
   */
  plugin.handleFormFields = function (context, settings) {

    // Ajax request to fill products related data.
    const $formElementQtoolsCommonConfig =  $(once('form-element--qtools-common-config', '.form-element--qtools-common-config', context));
    $formElementQtoolsCommonConfig.each(function () {
      var $element = $(this);
      var payload = $element.data('qtools-config-json');
      var $override_enabled = $('input[name="' + $element.attr('name') + '_override_enabled"]');
      var $checkbox = $('<input type="checkbox" title="' + payload.title + '" />');
      $checkbox.insertBefore($element);

      if ($override_enabled.val() === "1") {
        // Init current value as overridden to support
        // FAPI errors validation.
        $element.data('qtools-config-overridden-value', $element.val());
        plugin.useOverride($element, $override_enabled, $checkbox);
      }
      else {
        plugin.useFallback($element, $override_enabled, $checkbox);
      }

      $checkbox.click(function () {
        // At this point prop already changed so we can just use it.
        var value = $(this).prop('checked');
        if (value) {
          plugin.useOverride($element, $override_enabled, $checkbox);
        }
        else {
          plugin.useFallback($element, $override_enabled, $checkbox);
        }
      });

      $(this).addClass('processed');
    });
  };

  /**
   * Enables use of overridden config.
   */
  plugin.useOverride = function ($element, $override_enabled, $checkbox) {
    // Check override checkbox.
    $element.parent().removeClass('form-disabled');
    $checkbox.prop('checked', true);
    $override_enabled.val("1");

    // Enable element.
    $element.attr('disabled', false);

    // Set overridden value.
    $element.val($element.data('qtools-config-overridden-value'));
  };

  /**
   * Enables use of fallback config.
   */
  plugin.useFallback = function ($element, $override_enabled, $checkbox) {
    // Save overridden value.
    $element.data('qtools-config-overridden-value', $element.val());

    // Set fallback value.
    $element.val($element.data('qtools-config-fallback-value'));

    // Disable element.
    $element.attr('disabled', true);

    // Uncheck overridden checkbox.
    $element.parent().addClass('form-disabled');
    $checkbox.prop('checked', false);
    $override_enabled.val("0");
  };

  // Register behavior.
  Drupal.behaviors.qtoolsCommonFormElementQtoolsCommonConfig = {
    attach: function (context, settings) {
      plugin.handleFormFields(context, settings)
    }
  };

}(Drupal, jQuery, once));
