<?php

namespace Drupal\qtools_common\Plugin\Field\FieldWidget;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\link\Plugin\Field\FieldWidget\LinkWidget;

/**
 * Link field widget plugin with option to open in new tab.
 *
 * @FieldWidget(
 *   id = "qtools_common_link_target_field_widget",
 *   label = @Translation("Link with target"),
 *   field_types = {
 *     "link"
 *   }
 * )
 */
class LinkTargetFieldWidget extends LinkWidget {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);
    $item = $items[$delta];
    $options = $item->get('options')->getValue();

    $default_value = !empty($options['attributes']['target']) ? $options['attributes']['target'] : '';
    $element['options']['attributes']['target'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Open in a new tab.'),
      '#default_value' => !empty($default_value),
      '#element_validate' => [[static::class, 'validateTargetAttribute']],
    ];
    return $element;
  }

  /**
   * Set attribute value to '_blank' instead of checbox's 1.
   */
  public static function validateTargetAttribute(&$element, FormStateInterface $form_state, $form) {
    $values = $form_state->getValues();
    if (!empty($element['#value'])) {
      NestedArray::setValue($values, $element['#parents'], '_blank');
    }
    else {
      NestedArray::unsetValue($values, $element['#parents']);
    }
    $form_state->setValues($values);
  }

}
