<?php

namespace Drupal\qtools_common;

use Drupal\Core\Database\Connection;
use Drupal\Core\Session\AccountProxyInterface;

/**
 * Class ChangesTrackingService.
 *
 * @package Drupal\qtools_common
 */
class QToolsConfigurationChangesTrackingService {

  /**
   * Drupal\Core\Database\Connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * Current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  const TABLE_LOGS = 'qtools_common_configuration_changes_log';

  const OVERRIDDEN_STATUS_YES = 'o';
  const OVERRIDDEN_STATUS_NO = 'd';
  const VALUE_SEPARATOR = '|';

  /**
   * Constructor.
   */
  public function __construct(
    Connection $connection,
    AccountProxyInterface $currentUser
  ) {
    $this->connection = $connection;
    $this->currentUser = $currentUser;
  }

  /**
   * Tracks change in configuration.
   */
  public function trackConfigChange($config_key, $old_value, $old_overridden_status, $new_value, $new_overridden_status) {
    $record = [
      'uid' => $this->currentUser->id(),
      'config_key' => $config_key,
      'old_value' => $this->stringValue($old_value, $old_overridden_status),
      'new_value' => $this->stringValue($new_value, $new_overridden_status),
      'timestamp' => time(),
    ];

    $this->connection->insert(static::TABLE_LOGS)
      ->fields($record)
      ->execute();
  }

  /**
   * Converts real value into string representation.
   *
   * @return string
   *   String representation of the value.
   */
  protected function stringValue($value, $overridden_status) {
    return ($overridden_status ? static::OVERRIDDEN_STATUS_YES : static::OVERRIDDEN_STATUS_NO) .
      static::VALUE_SEPARATOR .
      (string) $value;
  }

}

