<?php

namespace Drupal\qtools_common;

use Drupal\Core\Render\Element;

/**
 * Class QToolsFormHelper.
 *
 * @package Drupal\qtools
 */
abstract class QToolsFormHelper {

  /**
   * Filters children on the element, leaving ones that match filters.
   *
   * @deprecated
   *
   * @return array
   *   List of matched children on a form element.
   */
  public static function filterChildren(&$element, array $filters, $and = TRUE, $invert = FALSE, &$children = NULL) {
    if ($children === NULL) {
      $children = [];
    }
    foreach (Element::children($element) as $key) {
      // If child has own children - travers deeper.
      $sub_children = Element::children($element[$key]);
      if (!empty($sub_children)) {
        static::filterChildren($element[$key], $filters, $and, $invert, $children);
      }
      elseif (static::elementMatchFilters($element[$key], $filters, $and, $invert)) {
        $children[] = &$element[$key];
      }
    }

    return $children;
  }

  /**
   * Check if element matches set of filters.
   *
   * @deprecated
   */
  protected static function elementMatchFilters(&$element, $filters, $and, $invert) {
    $match = $and ? TRUE : FALSE;

    foreach ($filters as $filter) {
      $match = $and ?
        $match && static::elementMatchFilterInner($element, $filter) :
        $match || static::elementMatchFilterInner($element, $filter);
    }

    return $invert ? !$match : $match;
  }

  /**
   * Check if element matches one filter.
   *
   * @deprecated
   */
  protected static function elementMatchFilterInner(&$element, $filter) {

    list($op, $key) = $filter;
    $value = isset($filter[2]) ? $filter[2] : NULL;
    $invert = isset($filter[3]) ? $filter[3] : FALSE;
    switch ($op) {
      case 'isset':
        $match = isset($element[$key]);
        break;

      case 'empty':
        $match = empty($element[$key]);
        break;

      default:
        $match = isset($element[$key]) && $element[$key] == $value;
        break;
    }

    return $invert ? !$match : $match;
  }

}
