<?php

namespace Drupal\qtools_common\Utility;

/**
 * Trims text considering HTML tags.
 *
 * This is modified version of the Class from stackoverflow.
 * all credits to original authors.
 *
 * @see https://stackoverflow.com/a/16584383
 * @see https://stackoverflow.com/a/62457155
 */
class HtmlTrimmer {

  /**
   * Flag is we reach the length limit.
   *
   * @var bool
   */
  protected $reachedLimit = FALSE;

  /**
   * Current total length.
   *
   * @var int
   */
  protected $totalLen = 0;

  /**
   * List of DOM nodes to remove.
   *
   * @var array
   */
  protected $toRemove = [];

  /**
   * Trims given html to specified length of visible symbols.
   */
  public static function trim(string $html, int $maxLen = 600): string {

    libxml_use_internal_errors(TRUE);

    $dom = new \DOMDocument();
    $dom->loadHTML('<?xml encoding="UTF-8">' . $html, LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);

    $instance = new static();
    $toRemove = $instance->walk($dom, $maxLen);

    // Remove any nodes that exceed limit.
    foreach ($toRemove as $child) {
      $child->parentNode->removeChild($child);
    }

    // Log errors? atm seems like just useless spam because of small HTML
    // error in WYSIWYG.
    libxml_get_errors();

    return str_replace('<?xml encoding="UTF-8">', '', $dom->saveHTML());
  }

  /**
   * Walks document checking is given node fits allowed limit or not.
   */
  protected function walk(\DOMNode $node, int $maxLen): array {

    if ($this->reachedLimit) {
      $this->toRemove[] = $node;
    }
    else {
      // Only text nodes should have text,
      // so do the splitting here.
      if ($node instanceof \DOMText) {
        $this->totalLen += $nodeLen = mb_strlen($node->nodeValue);

        // Use mb_strlen / mb_substr for UTF-8 support.
        if ($this->totalLen > $maxLen) {
          $node->nodeValue = mb_substr($node->nodeValue, 0, $nodeLen - ($this->totalLen - $maxLen)) . '...';
          $this->reachedLimit = TRUE;
        }
      }

      // If node has children, walk its child elements.
      if (isset($node->childNodes)) {
        foreach ($node->childNodes as $child) {
          $this->walk($child, $maxLen);
        }
      }
    }

    return $this->toRemove;
  }

}
