<?php

namespace Drupal\qtools_common\Utility;

use Drupal\qtools_common\QToolsConfigHelper;

/**
 * Provides trait services configuration management with overrides.
 *
 * If service uses this trait, it must implement
 * "QToolsConfigurableInterface" and be tagged with
 *  tags:
 *   - { name: qtools_configurable_service }
 */
trait QToolsConfigTrait {

  /**
   * Gets config value.
   *
   * @throws \Exception
   */
  public function config(string $key): mixed {
    $defaults = static::getConfigTraitDefaults();
    $bundle = static::getConfigTraitBundle();

    // Check if this key exists among defaults.
    if (!array_key_exists($key, $defaults)) {
      $error = t('Config key [@key] not found in defaults for [@name].', [
        '@key' => $key,
        '@name' => $bundle,
      ]);
      throw new \Exception($error);
    }

    return QToolsConfigHelper::get($bundle . '][' . $key, $defaults[$key]);
  }

}
