<?php

namespace Drupal\qtools_common\Utility;

/**
 * Provides trait for methods objects debugging.
 */
trait QToolsDebugTrait {

  /**
   * Prints debug message taking into account place it was called.
   */
  public function l($data = 'called', $name = '', $call_step = 1) {
    // Get called class name.
    $tpm = explode('\\', get_class($this));
    $object = end($tpm);
    $bt = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS);
    $call = $bt[$call_step];
    $name .= '[ ' . $object . ' ] ' . $call['class'] . $call['type'] . $call['function'] . '( ' . $bt[$call_step - 1]['line'] . ' )';
    $logger = 'dpm';
    if (function_exists($logger)) {
      $logger($data, $name);
    }
  }

  /**
   * Starts/measures timer and return its key.
   */
  public function timer($key = NULL, $log = TRUE) {
    $timers = &drupal_static('QToolsDebugTrait:timers', []);

    // Create new timer if no key passed.
    if ($key === NULL) {
      $key = uniqid(TRUE);
      $timers[$key] = microtime(TRUE);
      return $key;
    }
    else {
      // Return elapsed time on given timer.
      $elapsed = microtime(TRUE) - $timers[$key];
      if ($log) {
        $this->l($elapsed, $key, 2);
      }
      return $elapsed;
    }
  }

  /**
   * Measure and output time spent on callable execution.
   */
  public function __call($name, $arguments) {

    // Check if this is performance test call.
    if (strpos($name, 'p_') === 0) {
      $name = substr($name, 2);

      // Check if method in question exists.
      if (method_exists($this, $name)) {
        $timer = $this->timer();
        $result = call_user_func_array([$this, $name], $arguments);
        $this->l($this->timer($timer), 'elapsed time: ', 2);
        return $result;
      }
    }

    // If we get here throw exception.
    throw new \Exception('Function ' . $name . 'not found');
  }

}
