<?php

namespace Drupal\qtools_common\Utility;

/**
 * Provides helper to filter slugify text.
 *
 * @ingroup utility
 */
class Slug {

  protected static $frechReplacements = [
    'à' => 'a',
    'â' => 'a',
    'á' => 'a',
    'ä' => 'a',

    'é' => 'e',
    'è' => 'e',
    'ê' => 'e',
    'ë' => 'e',

    'í' => 'i',
    'ï' => 'i',
    'î' => 'i',

    'ó' => 'o',
    'ô' => 'o',

    'ú' => 'u',
    'û' => 'u',
    'ü' => 'u',

    'ç' => 'c',
    'ẃ' => 'w',
    'ý' => 'y',
    'ÿ' => 'y',
  ];

  /**
   * Convert string to slug.
   *
   * @return string
   *   Cleaned up version of the HTML attributes.
   */
  public static function slugify($text, $replacement = '-') {
    return strtolower(trim(preg_replace('/[^A-Za-z0-9-]+/', $replacement, $text), $replacement));
  }

  /**
   * Convert string to slug.
   *
   * @return string
   *   Cleaned up version of the HTML attributes.
   */
  public static function frenchText($text, $replacement = ' ') {
    return strtolower(trim(preg_replace('/[^A-Za-z0-9àâäáèéêëíîïôœùûüÿẃçÀÂÄÈÉÊËÎÏÔŒÙÛÜŸÇ -]+/u', $replacement, $text)));
  }

  /**
   * Prepare clean id from possibly French symbols.
   *
   * There is more fancy ways to do this, but I like to have a list of
   * accented chars on my hand :).
   *
   * @return string
   *   Frenchliterated text.
   */
  public static function frenchliterate($text, $keep_case = FALSE) {
    if (!$keep_case) {
      $text = mb_strtolower($text);
    }

    return strtr($text, static::$frechReplacements);
  }

}
