<?php

namespace Drupal\qtools_common\Utility;

use Drupal\Core\Database\Database;

/**
 * Provides functions for UID generation.
 *
 * @ingroup utility
 */
class Unique {

  const UID_TABLE = 'qtools_common_uid';

  /**
   * Returns unique id.
   *
   * Returns unique id guarantee it to be unique within the site
   * as it uses DB serial to sync between requests.
   *
   * @return int
   *   Unique id.
   */
  public static function id($length = NULL, $pad_string = '0', $pack = NULL) {
    $connection = Database::getConnection();
    if ($connection->schema()->tableExists(static::UID_TABLE)) {
      $id = Database::getConnection()->insert(static::UID_TABLE)
        ->fields(['id'], [NULL])
        ->execute();

      // Clean up each 1000 records.
      if (($id % 1000) == 0) {
        $connection->delete(static::UID_TABLE)
          ->condition('id', $id, '<')
          ->execute();
      }
    }
    else {
      $id = 0;
    }

    if (!empty($pack) && $pack <= 36) {
      $id = static::pack($id, $pack);
    }

    if (!empty($length)) {
      $id = str_pad($id, $length, $pad_string, STR_PAD_LEFT);
    }

    return $id;
  }

  /**
   * Pack numeric id.
   *
   * @return string
   *   Compressed id with string literals.
   */
  public static function pack($id, $to_base = 36) {
    return base_convert($id, 10, $to_base);
  }

  /**
   * Unpack numeric id.
   *
   * @return int
   *   Uncompressed id with only digits.
   */
  public static function unpack($id, $from_base = 36) {
    return base_convert($id, $from_base, 10);
  }

}
